@@ -518,12 +518,24 @@ int parse_descriptor_binding(Instance * instance, DescriptorBinding * binding, P
                 }
             }
 
+            VkImageViewType image_view_type = VK_IMAGE_VIEW_TYPE_2D;
+
+            if (PyObject * temp = PyDict_GetItemString(item, "cube")) {
+                if (!PyBool_Check(temp)) {
+                    PyErr_Format(PyExc_ValueError, "cube");
+                    return -1;
+                }
+                if (PyObject_IsTrue(temp)) {
+                    image_view_type = VK_IMAGE_VIEW_TYPE_CUBE;
+                }
+            }
+
             binding->image.image_view_create_info_array[i] = {
                 VK_STRUCTURE_TYPE_IMAGE_VIEW_CREATE_INFO,
                 NULL,
                 0,
                 image->image,
-                VK_IMAGE_VIEW_TYPE_2D,
+                image_view_type,
                 image->format,
                 swizzle,
                 {VK_IMAGE_ASPECT_COLOR_BIT, level, levels, layer, layers},
